#
# ademco.event - description of Ademco system events
#
# Copyright 2013-2016 Perry The Cynic. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#	http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import time

ARM_AWAY	= '07'	# arm partition
ARM_STAY	= 'B7'	# arm partition
DISARM		= '08'	# disarm partition
FAULT		= 'F5'	# zone faulting
UNFAULT		= 'F6'	# zone fault restore


#
# Event table (indexed by hex*2 code)
#
EVTYPE = {
	'01':	('alarm',	'Fire Alarm'),
	'02':	('alarm',	'Fire Alarm Restore', '01'),
	'03':	('trouble',	'Trouble'),
	'04':	('trouble',	'Trouble Restore', '03'),
	'05':	('bypass',	'Bypass'),
	'06':	('bypass',	'Bypass Restore', '05'),
	'07':	('open',	'Close (arm)'),
	'08':	('open',	'Open (disarm)'),
	'0D':	('system',	'Man. Trigger Test Report'),
	'0E':	('system',	'Send a Power-up Report'),
	'0F':	('alarm',	'Exit Error By User'),

	'11':	('alarm',	'Duress'),
	'12':	('alarm',	'Duress Restore', '11'),
	'13':	('trouble',	'Telco Line 1 Trouble'),
	'14':	('trouble',	'Telco Line 1 Trouble Restore', '13'),
	'15':	('bypass',	'Bell 1 Disable (bypass)'),
	'16':	('bypass',	'Bell 1 Bypass Restore', '15'),
	'17':	('open',	'Remote Close (arm)'),
	'18':	('open',	'Remote Open (disarm)'),
	'19':	('system',	'Pager Failed'),
	'1A':	('system',	'Pager Restore', '19'),
	'1B':	('system',	'AC Loss'),
	'1C':	('system',	'AC Restore', '1B'),
	'1D':	('system',	'Periodic Test Report'),
	'1E':	('system',	'Except Skd Chg'),
	'1F':	('alarm',	'Exit Error By Zone'),

	'21':	('alarm',	'Silent'),
	'22':	('alarm',	'Silent Restore', '21'),
	'23':	('trouble',	'Telco Line 2 Trouble'),
	'24':	('trouble',	'Telco Line 2 Trouble Restore', '23'),
	'25':	('bypass',	'Bell 2 Disable (bypass)'),
	'26':	('bypass',	'Bell 2 Bypass Restore', '25'),
	'27':	('open',	'Quick Arm (close)'),
	'29':	('system',	'Low Battery'),
	'2A':	('system',	'Low Battery Restore', '29'),
	'2B':	('open',	'Access Denial (general)'),
	'2C':	('trouble',	'AC Loss at ACS Module'),
	'2D':	('system',	'Walk Test'),
	'2E':	('system',	'Access Skd Chg'),
	'2F':	('system',	'Fire Walk Test'),

	'31':	('alarm',	'Audible'),
	'32':	('alarm',	'Audible Restore', '31'),
	'33':	('trouble',	'Earth Ground Trouble'),
	'34':	('trouble',	'Earth Ground Trouble Restore'),
	'35':	('bypass',	'Aux Relay Disable (bypass)'),
	'36':	('bypass',	'Aux Relay Disable Restore', '35'),
	'37':	('open',	'Keyswitch Close (arm)'),
	'38':	('open',	'Keyswitch Open (disarm)', '37'),
	'39':	('system',	'Battery Charger Failed'),
	'3A':	('system',	'Battery Charger Restore', '39'),
	'3B':	('trouble',	'Door Prop Open'),
	'3C':	('system',	'Low Battery at ACS Module'),
	'3D':	('system',	'Walk Test Exit'),
	'3E':	('system',	'Send a Power-up Report'),
	'3F':	('test',	'Fire Walk Test Exit'),

	'41':	('alarm',	'Perimeter'),
	'42':	('alarm',	'Perimeter Restore'),
	'43':	('trouble',	'Zone/Sensor Supervisor Alarm'),
	'44':	('trouble',	'Zone/Sensor Supervisor Alarm Restore', '43'),
	'45':	('bypass',	'Dialer Disable (bypass)'),
	'46':	('bypass',	'Dialer Bypass Restore', '45'),
	'47':	('open',	'Partial Arm (close)'),
	'48':	('system',	'Callback Requested'),
	'4B':	('trouble',	'Door Prop Open Restore', '3B'),
	'4C':	('bypass',	'Access Point Bypass'),
	'4D':	('all',		'Event Log 50% Full'),
	'4E':	('all',		'Program Changed'),

	'51':	('alarm',	'Interior'),
	'52':	('alarm',	'Interior Restore', '51'),
	'53':	('trouble',	'Exp. Module Tamper'),
	'54':	('trouble',	'Exp. Module Tamper Restore', '53'),
	'55':	('bypass',	'Vent Zone (bypass)'),
	'56':	('bypass',	'Vent Zone Bypass Restore', '55'),
	'59':	('system',	'Battery Test Fail'),
	'5A':	('system',	'Battery Test Restore', '59'),
	'5B':	('system',	'Access Granted'),
	'5C':	('all',		'Reset At Module'),
	'5D':	('all',		'Event Log 90% Full'),
	'5E':	('open',	'Auto-arm Fail'),
	'5F':	('alarm',	'Send A Cancel Trigger'),

	'61':	('alarm',	'24 Hour Zone'),
	'62':	('alarm',	'24 Hour Restore', '61'),
	'63':	('trouble',	'RF Sensor Supervisor'),
	'64':	('trouble',	'RF Sensor Supervisor Restore', '63'),
	'65':	('system',	'ACS Test Entry'),
	'66':	('system',	'ACS Test Exit', '65'),
	'67':	('open',	'Auto Close (arm)'),
	'68':	('open',	'Auto Open (disarm)', '67'),
	'69':	('trouble',	'Expander Module Fail'),
	'6A':	('open',	'Expander Module Fail'),
	'6B':	('trouble',	'Egress Denied (General)'),
	'6C':	('system',	'Access Point Relay Supervision Fail'),
	'6D':	('system',	'Event Log Overwrite'),
	'6E':	('system',	'Off Normal Report'),
	'6F':	('test',	'Begin Drill (Fire)'),

	'71':	('alarm',	'Day/Nite Alarm'),
	'72':	('alarm',	'Day/Note Alarm Restore'),
	'73':	('trouble',	'RPM Sensor Super'),
	'74':	('trouble',	'RPM Sensor Super Restore'),
	'76':	('all',		'Engineer Reset'),
	'77':	('system',	'Log Dialer Shutdown Restore', '78'),
	'78':	('system',	'Log Dialer Shutdown'),
	'79':	('all',		'System Shutdown'),
	'7A':	('all',		'System Shutdown Restore'),
	'7B':	('alarm',	'Door Forced Open'),
	'7C':	('trouble',	'Self Test Fail At Module'),
	'7D':	('all',		'Event Log Reset'),
	'7E':	('fire',	'Point Tested Ok'),
	'7F':	('test',	'End Drill (Fire)', '6F'),

	'81':	('alarm',	'Entry/Exit Alarm'),
	'82':	('alarm',	'Entry/Exit Alarm Restore', '81'),
	'83':	('trouble',	'ECP Relay Trouble'),
	'84':	('trouble',	'ECP Relay Trouble Restore', '83'),
	'87':	('system',	'Log System Shutdown Restore', '88'),
	'88':	('system',	'Log System Shutdown'),
	'89':	('trouble',	'RF Low Battery'),
	'8A':	('trouble',	'RF Low Battery Restore', '89'),
	'8B':	('alarm',	'Door Forced Open Restore', '7B'),
	'8C':	('bypass',	'Access Point Dsm Shunt'),
	'8D':	('all',		'Time Clock Reset'),
	'8E':	('system',	'Point Not Tested (Burglary)'),
	'8F':	('test',	'Point Not Tested (Fire)'),

	'91':	('alarm',	'Poll Loop Short'),
	'92':	('alarm',	'Poll Loop Short Restore', '91'),
	'93':	('trouble',	'Polling Loop Short'),
	'94':	('trouble',	'Polling Loop Short Restore', '93'),
	'95':	('bypass',	'ACS Relay/Trigger Disable'),
	'96':	('bypass',	'ACS Relay/Trigger Enable', '95'),
	'97':	('bypass',	'ACS Reader Disable'),
	'98':	('bypass',	'ACS Reader Enable', '97'),
	'99':	('alarm',	'ACS Zone Alarm'),
	'9A':	('alarm',	'ACS Zone Alarm Restore', '99'),
	'9B':	('open',	'Egress Granted'),
	'9C':	('bypass',	'Access Point Dsm Unshunt', '8C'),
	'9D':	('all',		'Time Clock Wrong'),
	'9E':	('alarm',	'Recent Close By User'),
	'9F':	('trouble',	'Remote PS Loss Restore'),

	'A1':	('alarm',	'Expander Module Fail'),
	'A2':	('alarm',	'Expander Module Fail Restore', 'A1'),
	'A3':	('trouble',	'Expander Module Trouble'),
	'A4':	('trouble',	'Expander Module Trouble Restore', 'A3'),
	'A5':	('bypass',	'ACS Zone Shunt'),
	'A6':	('bypass',	'ACS Zone Shunt Restore', 'A5'),
	'A7':	('trouble',	'Access Point RTE Trouble'),
	'A8':	('trouble',	'Access Point RTE Trouble Restore', 'A7'),
	'A9':	('trouble',	'Access Point DSM Trouble'),
	'AA':	('trouble',	'Access Point DSM Trouble Restore', 'A9'),
	'AB':	('bypass',	'Access Point RTE Shunt'),
	'AC':	('bypass',	'Access Point RTE Shunt Restore', 'AB'),
	'AD':	('system',	'Log Program Mode Entry'),
	'AE':	('alarm',	'Listen-in To Follow'),

	'B1':	('alarm',	'Non-burglar Alarm'),
	'B2':	('alarm',	'Non-burglar Alarm'),
	'B3':	('trouble',	'Sensor Tamper'),
	'B4':	('trouble',	'Sensor Tamper Restore', 'B3'),
	'B5':	('trouble',	'Cross-zoning Trouble'),
	'B6':	('trouble',	'Cross-zoning Trouble Restore', 'B5'),
	'B7':	('open',	'Arm Stay (close)'),
	'BB':	('all',		'ACS Program Entry'),
	'BC':	('trouble',	'AC Loss Restored ACS Module', 'BB'),
	'BD':	('system',	'Program Mode Exited'),
	'BE':	('trouble',	'Point Tested Failed'),

	'C1':	('alarm',	'Smoke Alarm'),
	'C2':	('alarm',	'Smore Alarm Restore', 'C1'),
	'C3':	('trouble',	'Fire Trouble'),
	'C4':	('trouble',	'Fire Trouble Restore', 'C3'),
	'C5':	('trouble',	'CO Alarm'),
	'C6':	('trouble',	'CO Alarm Restore', 'C5'),
	'C7':	('open',	'Fail To close (arm)'),
	'C8':	('open',	'Fail To Open (disarm)'),
	'C9':	('trouble',	'Smoke Detector High Sensitivity'),
	'CA':	('trouble', 'Smoke Detector High Sensitivity Restore', 'C9'),
	'CB':	('all',		'ACS Program Exit'),
	'CC':	('trouble',	'Low Battery Restored ACS Module', '3C'),
	'CD':	('system',	'User Code Added'),

	'D1':	('alarm',	'Water Flow Alarm'),
	'D2':	('alarm',	'Water Flow Alarm Restore', 'D1'),
	'D3':	('all',		'Fail To Communicate'),
	'D4':	('all',		'Communications Restore', 'D3'),
	'D7':	('open',	'Late Close (arm)'),
	'D8':	('open',	'Late Open'),
	'D9':	('trouble',	'Smoke Detector Low Sensitivity'),
	'DA':	('trouble',	'Smoke Detector Low Sensitivity Restore', 'D9'),
	'DB':	('all',		'ACS Threat Change'),
	'DC':	('bypass',	'Access Point Unbypass', '4C'),
	'DD':	('system',	'User Code Deleted'),

	'E1':	('alarm',	'Zone/Sensor Supervisor Alarm'),
	'E2':	('alarm',	'Zone/Sensor Supervisor Alarm Restore', 'E1'),
	'E3':	('trouble',	'Bell 1 Trouble'),
	'E4':	('trouble',	'Bell 1 Trouble Restore', 'E3'),
	'E7':	('open',	'Early Close (arm)'),
	'E8':	('open',	'Early Open'),
	'E9':	('trouble',	'Intrusion Detection High Sensitivity'),
	'EA':	('trouble',	'Intrusion Detection High Sensitivity Restore', 'E9'),
	'EB':	('alarm',	'Duress Access Grant'),
	'EC':	('trouble',	'Access Point Relay Supervisor Restore', '6C'),
	'ED':	('system',	'User Code Changed'),

	'F3':	('trouble',	'Bell 2 Trouble'),
	'F4':	('trouble',	'Bell 2 Trouble Restore', 'F3'),
	'F5':	('trouble',	'Faults'),
	'F6':	('trouble', 'Fault Restores', 'F6'),
	'F8':	('trouble', 'Dialer Queue Full'),
	'F9':	('trouble', 'PIR Detector Lo Sensitivity'),
	'FA':	('alarm',	'Intrusion Detection Low Sensitivity'),
	'FB':	('trouble',	'Duress Egress Grant'),
	'FC':	('trouble',	'Selftest Restored ACS Module', 'FB'),
	'FD':	('system',	'Fail to Print'),
	'FE':	('system',	'Fail to Print Restore', 'FD'),
}


class SystemEvent(object):

	def __init__(self, system, body):
		self.system = system
		self.code = body[0:2]
		desc = EVTYPE.get(self.code) or ('trouble', 'Unknown event code %s' % self.code)
		self.kind = desc[0]
		self.meaning = desc[1]
		self.zone = int(body[2:5])
		self.uid = int(body[5:8])
		self.partition = int(body[8:9])
		self.time = time.strptime(body[9:19], "%M%H%d%m%y")

	def __str__(self):
		return "<%s %s:%s Z%d P%s U%d @%s>" % (
			self.kind, self.code, self.meaning, self.zone, self.partition, self.uid,
			time.asctime(self.time)
		)
